# A constant used in websocket handshake
# Note, this isn't anything that needs to be secured.
# Sett https://tools.ietf.org/html/rfc6455#page-7
WS_KEY = b"258EAFA5-E914-47DA-95CA-C5AB0DC85B11"
# Standard since 2011
# https://tools.ietf.org/html/rfc6455
WS_VERSION = 13


from __future__ import unicode_literals

from .frame_parser import FrameParser
from .message import Message
from .response import Response


class WebsocketStream(object):
    """
    Parses a stream of data in to Headers and logical Websocket
    frames.

    """

    def __init__(self):
        self.frame_parser = FrameParser()
        self._parsed_response = False
        self._frames = []

    def feed(self, data):
        """Feed in data from a socket to yield 0 or more frames."""
        # This combines fragmented frames in to a single frame
        iter_frames = iter(self.frame_parser.feed(data))

        if not self._parsed_response:
            # Process status line and headers from frame parser
            header_data = next(iter_frames, None)
            if header_data is None:
                return
            self._parsed_response = True
            yield Response(header_data)

        # Process incoming frames
        for frame in iter_frames:
            if frame.is_control:
                # Control messages are never fragmented
                # And may be sent in the middle of a multi-part message
                yield Message.build([frame])
            else:
                # May be fragmented
                self._frames.append(frame)
                if frame.fin:
                    # Combine any multi part frames in to a single
                    # Message
                    yield Message.build(self._frames)
                    del self._frames[:]



"""
Websocket status codes

https://tools.ietf.org/html/rfc6455#section-7.4

"""

class Status(object):
    NORMAL = 1000
    GOING_AWAY = 1001
    PROTOCOL_ERROR = 1002

    BAD_DATA = 1007
    POLICY_VIOLATION = 1008
    MESSAGE_TOO_LARGE = 1009
    EXTENSION_FAILED = 1010

    UNEXPECTED_CONDITION = 1011



from __future__ import unicode_literals


class Error(Exception):
    def __init__(self, msg, *args, **kwargs):
        error_msg = msg.format(*args, **kwargs)
        super(Error, self).__init__(error_msg)


class FrameBuildError(Error):
    pass


class HandshakeError(Error):
    pass


class WebsocketError(Error):
    pass


class ProtocolError(Error):
    pass


class PayloadTooLarge(ProtocolError):
    pass


class WebSocketInUse(Error):
    pass


class TransportFail(Error):
    pass


class WebSocketClosed(Error):
    pass
